SystemOrganization addCategory: #'Bioloid-Core'!
SystemOrganization addCategory: #'Bioloid-Core-Protocol'!

Object subclass: #Bioloid
	instanceVariableNames: 'motors protocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bioloid-Core'!

!Bioloid methodsFor: 'connection' stamp: 'sgb 10/24/2012 16:24'!
connectOnPort: portName
protocol connectOnPort: portName
! !

!Bioloid methodsFor: 'connection' stamp: 'Richo 9/19/2012 18:17'!
disconnect
	protocol disconnect! !

!Bioloid methodsFor: 'accessing' stamp: 'sgb 11/12/2012 16:50'!
getDataFor: aBioloidMotor 
	^protocol getMotorState at: aBioloidMotor id. ! !

!Bioloid methodsFor: 'initialize-release' stamp: 'Richo 9/19/2012 18:05'!
initialize
	super initialize.
	protocol := BioloidProtocol new.
	motors := OrderedCollection new! !

!Bioloid methodsFor: 'connection' stamp: 'sgb 11/12/2012 16:03'!
initializeSampling
motors do: [:each | each activateSampling:true ].
	! !

!Bioloid methodsFor: 'connection' stamp: 'Richo 9/19/2012 18:17'!
isConnected
	^ protocol isConnected! !

!Bioloid methodsFor: 'accessing' stamp: 'Richo 9/19/2012 18:08'!
motors
	^ motors! !

!Bioloid methodsFor: 'private' stamp: 'sgb 10/29/2012 17:44'!
privateAttach: aMotor
	motors add: aMotor! !

!Bioloid methodsFor: 'private' stamp: 'sgb 10/29/2012 17:49'!
privateDetach: aMotor
	motors remove: aMotor! !

!Bioloid methodsFor: 'accessing' stamp: 'Richo 9/19/2012 18:07'!
protocol
	^ protocol! !

Object subclass: #BioloidMotor
	instanceVariableNames: 'bioloid id led goalPosition movingSpeed torqueLimit torqueEnable presentPosition presentSpeed presentLoad presentVoltage presentTemperature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bioloid-Core'!

!BioloidMotor class methodsFor: 'instance creation' stamp: 'Richo 9/19/2012 17:59'!
id: motorId
	^ self basicNew initializeWith: motorId! !

!BioloidMotor class methodsFor: 'instance creation' stamp: 'sgb 10/24/2012 16:41'!
new
self shouldNotImplement ! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 11/12/2012 15:45'!
activateSampling: aBoolean
	
	bioloid protocol executeServoSampling: aBoolean id: id
	
! !

!BioloidMotor methodsFor: 'attaching' stamp: 'sgb 11/12/2012 16:22'!
attachTo: aBioloid
	self isAttached
		ifTrue: [self error: ('{1} is already attached.' format: {self}).
			^ self].
	bioloid := aBioloid. 
	bioloid privateAttach: self.
	self activateSampling: true. 

! !

!BioloidMotor methodsFor: 'attaching' stamp: 'sgb 11/9/2012 16:09'!
detach
	self isAttached
		ifFalse: [self error: ('{1} is not attached' format: {self}).
				^self]. 
	self activateSampling:false.  
	bioloid privateDetach: self.	
	bioloid := nil. 
	! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/26/2012 15:54'!
goalPosition
	^ goalPosition ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/30/2012 17:58'!
goalPosition: aNumber
	
	goalPosition := aNumber min: 1023 max: 0.
	self isAttached
		ifTrue: [goalPosition := goalPosition rounded.
			bioloid protocol
				writeOn: id
				address: 30
				word: goalPosition]! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/29/2012 18:11'!
id
	^ id! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 11/9/2012 17:26'!
id: aSmallInteger 
	id := aSmallInteger.
	bioloid initializeSampling.! !

!BioloidMotor methodsFor: 'as yet unclassified' stamp: 'sgb 10/29/2012 18:23'!
initializeFrom: aBioloidMotor 
	self shouldBeImplemented! !

!BioloidMotor methodsFor: 'initialize-release' stamp: 'Richo 9/19/2012 18:00'!
initializeWith: aNumber
	id := aNumber.
	self initialize! !

!BioloidMotor methodsFor: 'testing' stamp: 'Richo 9/19/2012 18:04'!
isAttached
	^ bioloid notNil! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/26/2012 12:08'!
led 
	^ led ifNil: false! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/26/2012 12:01'!
led: aBoolean
	self isAttached
	ifTrue:
	[led:=aBoolean.
	bioloid protocol
		writeOn: id
		address: 25
		byte: aBoolean asBit]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/30/2012 17:05'!
movingSpeed
	^ movingSpeed ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/30/2012 17:59'!
movingSpeed: aNumber
	movingSpeed := aNumber min: 1023 max: 0.
	self isAttached
		ifTrue: [movingSpeed := movingSpeed rounded.
			bioloid protocol
				writeOn: id
				address: 32
				word: movingSpeed]! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 11/12/2012 17:42'!
presentLoad
	self updateData: (bioloid getDataFor: self). 
	^ presentLoad ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 11/12/2012 16:41'!
presentPosition
	self updateData: (bioloid getDataFor: self). 
	^ presentPosition ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 11/12/2012 17:31'!
presentSpeed
	self updateData: (bioloid getDataFor: self). 
	^ presentSpeed ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 11/12/2012 17:42'!
presentTemperature
	self updateData: (bioloid getDataFor: self). 
	^ presentTemperature ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 11/12/2012 17:42'!
presentVoltage
	self updateData: (bioloid getDataFor: self). 
	^ presentVoltage ifNil: [0]
! !

!BioloidMotor methodsFor: 'printing' stamp: 'Richo 9/19/2012 18:00'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(;
		print: id;
		nextPut: $)! !

!BioloidMotor methodsFor: 'private' stamp: 'sgb 10/29/2012 17:34'!
privateAttachTo: aBioloid 
	bioloid := aBioloid! !

!BioloidMotor methodsFor: 'private' stamp: 'sgb 10/29/2012 17:36'!
privateDetach
	bioloid := nil. 
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/30/2012 17:09'!
torqueEnable 
	^ torqueEnable ifNil: false! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/30/2012 17:08'!
torqueEnable: aBoolean
	self isAttached
	ifTrue:
	[torqueEnable:=aBoolean.
	bioloid protocol
		writeOn: id
		address: 24
		byte: aBoolean asBit]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/30/2012 17:07'!
torqueLimit
	^ torqueLimit ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/30/2012 17:59'!
torqueLimit: aNumber
	
	torqueLimit := aNumber min: 1023 max: 0.
	self isAttached
		ifTrue: [torqueLimit := torqueLimit rounded.
			bioloid protocol
				writeOn: id
				address: 34
				word: torqueLimit]! !

!BioloidMotor methodsFor: 'as yet unclassified' stamp: 'sgb 11/12/2012 17:19'!
updateData: anArray
	(anArray size= 5)ifFalse: [^self].
	presentPosition:=anArray first.
	presentSpeed:=anArray second.
	presentLoad:=anArray third.
	presentVoltage:=anArray fourth.
	presentTemperature:=anArray fifth. 
! !

Object subclass: #BioloidProtocol
	instanceVariableNames: 'port argsToRead queue motorDic steppingProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bioloid-Core-Protocol'!

!BioloidProtocol class methodsFor: 'constants-request commands' stamp: 'sgb 11/9/2012 16:04'!
rqActivateSampling
	^ 3! !

!BioloidProtocol class methodsFor: 'constants-request commands' stamp: 'Richo 9/19/2012 17:51'!
rqServoWriteByte
	^ 1! !

!BioloidProtocol class methodsFor: 'constants-request commands' stamp: 'Richo 9/19/2012 17:51'!
rqServoWriteWord
	^ 2! !

!BioloidProtocol class methodsFor: 'constants-request commands' stamp: 'sgb 11/7/2012 17:03'!
rsServoRead
	^ 1! !

!BioloidProtocol methodsFor: 'converting' stamp: 'Richo 9/19/2012 17:48'!
asArgument: byte
	^ byte bitOr: 2r10000000! !

!BioloidProtocol methodsFor: 'converting' stamp: 'Richo 9/19/2012 17:48'!
asCommand: byte
	^ byte! !

!BioloidProtocol methodsFor: 'connection' stamp: 'sgb 11/9/2012 15:09'!
connectOnPort: aString
	"If the port cant't be open it stores nil, this way we can know if it is
	connected by asking if port is nil.
	In windows we can't open a port using the port name, so I convert the
	string to an integer"
	self isConnected ifTrue: [^ self].
	port := SerialPort new baudRate: 57600.
	port := port
				openPort: (('COM*' match: aString)
						ifTrue: [aString asInteger]
						ifFalse: [aString]).
	self isConnected
		ifTrue: [self startSteppingProcess]! !

!BioloidProtocol methodsFor: 'connection' stamp: 'sgb 11/9/2012 15:09'!
disconnect
	self isConnected
		ifTrue: [self stopSteppingProcess.
			port close.
			port := nil.
			self initialize]! !

!BioloidProtocol methodsFor: 'receiving' stamp: 'sgb 11/9/2012 15:34'!
executeCommand
	| command |
	command := queue removeFirst.
	command caseOf: {
		[self class rsServoRead] -> [self executeServoRead].
		} otherwise: [Transcript show: ('Invalid command: {1}' format:  {command}); cr].
	argsToRead := -1.! !

!BioloidProtocol methodsFor: 'receiving' stamp: 'sgb 11/12/2012 17:17'!
executeServoRead
	| id value1 value2 array |
	id := queue removeFirst.
	array:=OrderedCollection new. 

	value1 := queue removeFirst.
	value2 := queue removeFirst.
	array add:(value1 bitOr: (value2 << 7)).

	value1 := queue removeFirst.
	value2 := queue removeFirst.
	array add:(value1 bitOr: (value2 << 7)).

	value1 := queue removeFirst.
	value2 := queue removeFirst.
	array add:(value1 bitOr: (value2 << 7)).

	value1 := queue removeFirst.
	value2 := queue removeFirst.
	array add:(value1).
	array add:(value2).

	motorDic at: id put: (array)
! !

!BioloidProtocol methodsFor: 'commands' stamp: 'sgb 11/9/2012 16:54'!
executeServoSampling: aBoolean id: id
	self send: {self asCommand: self class rqActivateSampling.
				self asArgument: id.
				self asArgument: aBoolean asBit} asByteArray.! !

!BioloidProtocol methodsFor: 'converting' stamp: 'sgb 11/9/2012 15:06'!
getArgument: byte
	^ byte bitAnd: 2r01111111! !

!BioloidProtocol methodsFor: 'converting' stamp: 'sgb 11/9/2012 15:07'!
getCommand: byte
	^ byte! !

!BioloidProtocol methodsFor: 'stepping' stamp: 'sgb 11/12/2012 16:49'!
getMotorState
	^motorDic! !

!BioloidProtocol methodsFor: 'initialize-release' stamp: 'sgb 11/12/2012 17:08'!
initialize
	super initialize.
	motorDic:= self newDictionaryFrom: 0 to: 18 withAll: (OrderedCollection  	new).
	
	queue := OrderedCollection new.
	argsToRead := -1! !

!BioloidProtocol methodsFor: 'converting' stamp: 'sgb 11/9/2012 15:07'!
isArgument: byte
	^ byte >> 7 = 1! !

!BioloidProtocol methodsFor: 'converting' stamp: 'sgb 11/9/2012 15:07'!
isCommand: byte
	^ byte >> 7 = 0! !

!BioloidProtocol methodsFor: 'connection' stamp: 'Richo 9/19/2012 17:45'!
isConnected
	^ port notNil! !

!BioloidProtocol methodsFor: 'initialize-release' stamp: 'sgb 11/9/2012 15:28'!
newDictionaryFrom: start to: stop withAll: value
	| dict |
	dict := Dictionary new.
	start to: stop do: [:each |
		dict at: each put: value].
	^ dict
! !

!BioloidProtocol methodsFor: 'receiving' stamp: 'sgb 11/7/2012 17:04'!
processInput
	| data |
	data := port readByteArray.
	"self haltIf: data notEmpty."
	data 
		do: [:inByte |
			(self isCommand: inByte)
				ifTrue: [self readCommand: inByte].
			(self isArgument: inByte)
				ifTrue: [self readArgument: inByte].

			argsToRead = 0
				ifTrue: [self executeCommand]]! !

!BioloidProtocol methodsFor: 'receiving' stamp: 'sgb 11/9/2012 15:34'!
readArgument: inByte
	| argument |
	argument := self getArgument: inByte.
	queue add: argument.
	argsToRead := argsToRead - 1! !

!BioloidProtocol methodsFor: 'receiving' stamp: 'sgb 11/9/2012 15:34'!
readCommand: inByte
	| command |
	queue := OrderedCollection new.
	command := self getCommand: inByte.
	queue add: command.
	self setArgsToReadFor: command.! !

!BioloidProtocol methodsFor: 'sending' stamp: 'Richo 9/19/2012 17:55'!
send: aByteArray
	Transcript show: aByteArray printString; cr.
	self isConnected ifFalse: [^ self].
	(port nextPutAll: aByteArray) < 0
		ifTrue: [
			"An error occurred, maybe the board was unplugged.
			Just in case, disconnect"
			self disconnect]! !

!BioloidProtocol methodsFor: 'receiving' stamp: 'sgb 11/12/2012 17:14'!
setArgsToReadFor: command
	command caseOf: {
		[self class rsServoRead] -> [argsToRead := 9].
		} otherwise: [Transcript show: 'Error - command: ', command asString; cr]
! !

!BioloidProtocol methodsFor: 'stepping' stamp: 'sgb 11/9/2012 15:12'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step.
			(Delay forMilliseconds: self stepTime) wait] repeat]
				forkNamed: self asString ! !

!BioloidProtocol methodsFor: 'stepping' stamp: 'sgb 11/9/2012 15:12'!
step
	[self processInput]
		on: Error
		do: [:err | 
			Transcript show: self asString , ': ' , err description; cr. 
			self initialize]! !

!BioloidProtocol methodsFor: 'stepping' stamp: 'sgb 11/9/2012 15:12'!
stepTime
	^ 1! !

!BioloidProtocol methodsFor: 'stepping' stamp: 'sgb 11/9/2012 15:12'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
	steppingProcess := nil! !

!BioloidProtocol methodsFor: 'commands' stamp: 'Richo 9/19/2012 17:55'!
writeOn: motorId address: address byte: value
	self send: {self asCommand: self class rqServoWriteByte.
				self asArgument: motorId.
				self asArgument: address.
				self asArgument: (value bitAnd: 2r01111111).
				self asArgument: (value bitAnd: 2r10000000) >> 7} asByteArray! !

!BioloidProtocol methodsFor: 'commands' stamp: 'Richo 9/19/2012 18:20'!
writeOn: motorId address: address word: value
	| byte1 byte2 |
	byte1 := value bitAnd: 2r11111111.
	byte2 := value >> 8.
	self send: {self asCommand: self class rqServoWriteWord.
				self asArgument: motorId.
				self asArgument: address.
				self asArgument: (byte1 bitAnd: 2r01111111).
				self asArgument: (byte1 bitAnd: 2r10000000) >> 7.
				self asArgument: (byte2 bitAnd: 2r01111111).
				self asArgument: (byte2 bitAnd: 2r10000000) >> 7} asByteArray! !
